@@ -10,6 +10,6 @@ SEXP dpoMatrix_solve(SEXP a);
 SEXP dpoMatrix_matrix_solve(SEXP a, SEXP b);
 SEXP dpoMatrix_dgeMatrix_solve(SEXP a, SEXP b);
 SEXP dpoMatrix_chol(SEXP x);
-double get_norm_sy(SEXP obj, char *typstr);
+double get_norm_sy(SEXP obj, const char *typstr);
 
 #endif
